// Copyright 2013 Canonical Ltd.  This software is licensed under the
// GNU Lesser General Public License version 3 (see the file COPYING).
//
// Test helpers to fake objects that go into, or come out of, the Azure API.

package gwacl

import (
    "math/rand"
)

type endpointParams struct {
    setname   string
    localport int
    name      string
    port      int
    protocol  string
}

func makeEndpoint(params endpointParams) *InputEndpoint {
    if params.setname == "" {
        params.setname = MakeRandomString(10)
    }
    if params.name == "" {
        params.name = MakeRandomString(10)
    }
    if params.protocol == "" {
        params.protocol = MakeRandomString(10)
    }
    if params.localport == 0 {
        params.localport = rand.Intn(65535)
    }
    if params.port == 0 {
        params.port = rand.Intn(65535)
    }

    return &InputEndpoint{
        LoadBalancedEndpointSetName: params.setname,
        LocalPort:                   params.localport,
        Name:                        params.name,
        Port:                        params.port,
        Protocol:                    params.protocol}
}

func makeLinuxProvisioningConfiguration() *LinuxProvisioningConfiguration {
    hostname := MakeRandomString(10)
    username := MakeRandomString(10)
    password := MakeRandomString(10)
    disableSSH := MakeRandomBool()

    return &LinuxProvisioningConfiguration{
        ConfigurationSetType: "LinuxProvisioningConfiguration",
        Hostname:             hostname,
        Username:             username,
        Password:             password,
        DisableSSHPasswordAuthentication: disableSSH}
}

func makeOSVirtualHardDisk() *OSVirtualHardDisk {
    HostCaching := BoolToString(MakeRandomBool())
    DiskLabel := MakeRandomString(10)
    DiskName := MakeRandomString(10)
    MediaLink := MakeRandomString(10)
    SourceImageName := MakeRandomString(10)

    return &OSVirtualHardDisk{
        HostCaching:     HostCaching,
        DiskLabel:       DiskLabel,
        DiskName:        DiskName,
        MediaLink:       MediaLink,
        SourceImageName: SourceImageName}
}

func makeRole() *Role {
    RoleSize := "ExtraSmall"
    RoleName := MakeRandomString(10)
    RoleType := "PersistentVMRole"
    config := makeLinuxProvisioningConfiguration()
    configset := []LinuxProvisioningConfiguration{*config}

    return &Role{
        RoleSize:          RoleSize,
        RoleName:          RoleName,
        RoleType:          RoleType,
        ConfigurationSets: configset}
}

func makeDnsServer() *DnsServer {
    name := MakeRandomString(10)
    address := MakeRandomString(10)

    return &DnsServer{
        Name:    name,
        Address: address}
}

func makeDeployment() *Deployment {
    Name := MakeRandomString(10)
    DeploymentSlot := "Staging"
    Label := MakeRandomString(10)
    VirtualNetworkName := MakeRandomString(10)
    role := makeRole()
    RoleList := []Role{*role}
    Dns := []DnsServer{*makeDnsServer()}

    return &Deployment{
        XMLNS:              XMLNS,
        XMLNS_I:            XMLNS_I,
        Name:               Name,
        DeploymentSlot:     DeploymentSlot,
        Label:              Label,
        RoleList:           RoleList,
        VirtualNetworkName: VirtualNetworkName,
        DNS:                Dns,
    }
}

func makeCreateStorageServiceInput() *CreateStorageServiceInput {
    ServiceName := MakeRandomString(10)
    Description := MakeRandomString(10)
    Label := MakeRandomString(10)
    AffinityGroup := MakeRandomString(10)
    Location := MakeRandomString(10)
    GeoReplicationEnabled := BoolToString(MakeRandomBool())
    ExtendedProperties := []ExtendedProperty{{
        Name:  MakeRandomString(10),
        Value: MakeRandomString(10)}}

    return &CreateStorageServiceInput{
        XMLNS:                 XMLNS,
        ServiceName:           ServiceName,
        Description:           Description,
        Label:                 Label,
        AffinityGroup:         AffinityGroup,
        Location:              Location,
        GeoReplicationEnabled: GeoReplicationEnabled,
        ExtendedProperties:    ExtendedProperties}
}
