// Copyright 2013 Canonical Ltd.  This software is licensed under the
// GNU Lesser General Public License version 3 (see the file COPYING).

package gwacl

import (
    . "launchpad.net/gocheck"
    "net/url"
)

type UtilsSuite struct{}

var _ = Suite(&UtilsSuite{})

func (suite *UtilsSuite) TestCheckPathComponents(c *C) {
    checkPathComponents("fred", "bob", "123", "a..b") // All okay.
    c.Check(
        func() { checkPathComponents("foo^bar") },
        PanicMatches, "'foo\\^bar' contains URI special characters")
    c.Check(
        func() { checkPathComponents("foo/bar") },
        PanicMatches, "'foo/bar' contains URI special characters")
    c.Check(
        func() { checkPathComponents("..") },
        PanicMatches, "'[.][.]' is not allowed")
}

type TestAddURLQueryParams struct{}

var _ = Suite(&TestAddURLQueryParams{})

func (*TestAddURLQueryParams) TestUsesBaseURL(c *C) {
    baseURL := "http://example.com"

    extendedURL := addURLQueryParams(baseURL, "key", "value")

    parsedURL, err := url.Parse(extendedURL)
    c.Assert(err, IsNil)
    c.Check(parsedURL.Scheme, Equals, "http")
    c.Check(parsedURL.Host, Equals, "example.com")
}

func (suite *TestAddURLQueryParams) TestEscapesParams(c *C) {
    key := "key&key"
    value := "value%value"

    uri := addURLQueryParams("http://example.com", key, value)

    parsedURL, err := url.Parse(uri)
    c.Assert(err, IsNil)
    c.Check(parsedURL.Query()[key], DeepEquals, []string{value})
}

func (suite *TestAddURLQueryParams) TestAddsToExistingParams(c *C) {
    uri := addURLQueryParams("http://example.com?a=one", "b", "two")

    parsedURL, err := url.Parse(uri)
    c.Assert(err, IsNil)
    c.Check(parsedURL.Query(), DeepEquals, url.Values{
        "a": {"one"},
        "b": {"two"},
    })
}

func (suite *TestAddURLQueryParams) TestAppendsRepeatedParams(c *C) {
    uri := addURLQueryParams("http://example.com?foo=bar", "foo", "bar")
    c.Check(uri, Equals, "http://example.com?foo=bar&foo=bar")
}

func (suite *TestAddURLQueryParams) TestAddsMultipleParams(c *C) {
    uri := addURLQueryParams("http://example.com", "one", "un", "two", "deux")
    parsedURL, err := url.Parse(uri)
    c.Assert(err, IsNil)
    c.Check(parsedURL.Query(), DeepEquals, url.Values{
        "one": {"un"},
        "two": {"deux"},
    })
}

func (suite *TestAddURLQueryParams) TestRejectsOddNumberOfParams(c *C) {
    defer func() {
        err := recover()
        c.Check(err, ErrorMatches, ".*got 1 parameter.*")
    }()
    addURLQueryParams("http://example.com", "key")
    c.Assert("This should have panicked", Equals, "But it didn't.")
}
